@@ -1,7 +1,7 @@
 /*
  * RemotePlugin.h - base class providing RPC like mechanisms
  *
- * Copyright (c) 2008-2010 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2008-2012 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -115,7 +115,7 @@ typedef int32_t key_t;
 
 // sometimes we need to exchange bigger messages (e.g. for VST parameter dumps)
 // so set a usable value here
-const int SHM_FIFO_SIZE = 64000;
+const int SHM_FIFO_SIZE = 512*1024;
 
 
 // implements a FIFO inside a shared memory segment
@@ -284,7 +284,7 @@ public:
 	// recursive lock
 	inline void lock()
 	{
-		if( !isInvalid() && ++m_lockDepth == 1 )
+		if( !isInvalid() && __sync_add_and_fetch( &m_lockDepth, 1 ) == 1 )
 		{
 #ifdef USE_QT_SEMAPHORES
 			m_dataSem.acquire();
@@ -297,16 +297,13 @@ public:
 	// recursive unlock
 	inline void unlock()
 	{
-		if( m_lockDepth > 0 )
+		if( __sync_sub_and_fetch( &m_lockDepth, 1) <= 0 )
 		{
-			if( --m_lockDepth == 0 )
-			{
 #ifdef USE_QT_SEMAPHORES
-				m_dataSem.release();
+			m_dataSem.release();
 #else
-				sem_post( m_dataSem );
+			sem_post( m_dataSem );
 #endif
-			}
 		}
 	}
 
@@ -418,7 +415,8 @@ private:
 			return;
 		}
 		lock();
-		while( _len > m_data->endPtr - m_data->startPtr )
+		while( isInvalid() == false &&
+				_len > m_data->endPtr - m_data->startPtr )
 		{
 			unlock();
 #ifndef LMMS_BUILD_WIN32
@@ -439,7 +437,7 @@ private:
 
 	void write( const void * _buf, int _len )
 	{
-		if( isInvalid() )
+		if( isInvalid() || _len > SHM_FIFO_SIZE )
 		{
 			return;
 		}
@@ -484,7 +482,7 @@ private:
 	sem_t * m_dataSem;
 	sem_t * m_messageSem;
 #endif
-	int m_lockDepth;
+	volatile int m_lockDepth;
 
 } ;
 
@@ -509,7 +507,8 @@ enum RemoteMessageIDs
 	IdSaveSettingsToFile,
 	IdLoadSettingsFromString,
 	IdLoadSettingsFromFile,
-	IdLoadPresetFromFile,
+	IdSavePresetFile,
+	IdLoadPresetFile,
 	IdDebugMessage,
 	IdUserBase = 64
 } ;
@@ -547,8 +546,7 @@ public:
 
 		message & addInt( int _i )
 		{
-			char buf[128];
-			buf[0] = 0;
+			char buf[32];
 			sprintf( buf, "%d", _i );
 			data.push_back( std::string( buf ) );
 			return *this;
@@ -556,8 +554,7 @@ public:
 
 		message & addFloat( float _f )
 		{
-			char buf[128];
-			buf[0] = 0;
+			char buf[32];
 			sprintf( buf, "%f", _f );
 			data.push_back( std::string( buf ) );
 			return *this;
@@ -610,7 +607,7 @@ public:
 		m_out = out;
 	}
 
-	void sendMessage( const message & _m );
+	int sendMessage( const message & _m );
 	message receiveMessage();
 
 	inline bool isInvalid() const
@@ -929,19 +926,21 @@ RemotePluginBase::~RemotePluginBase()
 
 
 
-void RemotePluginBase::sendMessage( const message & _m )
+int RemotePluginBase::sendMessage( const message & _m )
 {
 	m_out->lock();
 	m_out->writeInt( _m.id );
 	m_out->writeInt( _m.data.size() );
-	int j = 0;
+	int j = 8;
 	for( unsigned int i = 0; i < _m.data.size(); ++i )
 	{
 		m_out->writeString( _m.data[i] );
-		j += _m.data[i].size();
+		j += 4 + _m.data[i].size();
 	}
 	m_out->unlock();
 	m_out->messageSent();
+
+	return j;
 }
 
 
@@ -1102,13 +1101,15 @@ void RemotePluginClient::setShmKey( key_t _key, int _size )
 {
 #ifdef USE_QT_SHMEM
 	m_shmObj.setKey( QString::number( _key ) );
-	if( m_shmObj.attach() )
+	if( m_shmObj.attach() || m_shmObj.error() == QSharedMemory::NoError )
 	{
 		m_shm = (float *) m_shmObj.data();
 	}
 	else
 	{
-		debugMessage( "failed getting shared memory\n" );
+		char buf[64];
+		sprintf( buf, "failed getting shared memory: %d\n", m_shmObj.error() );
+		debugMessage( buf );
 	}
 #else
 	if( m_shm != NULL )
