@@ -1,7 +1,7 @@
 /*
  * midi.h - constants, structs etc. concerning MIDI
  *
- * Copyright (c) 2005-2010 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2012 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -81,10 +81,31 @@ enum MidiMetaEvents
 } ;
 
 
+enum MidiStandardControllers
+{
+	MidiControllerBankSelect = 0,
+	MidiControllerModulationWheel = 1,
+	MidiControllerBreathController = 2,
+	MidiControllerFootController = 4,
+	MidiControllerPortamentoTime = 5,
+	MidiControllerMainVolume = 7,
+	MidiControllerBalance = 8,
+	MidiControllerPan = 10,
+	MidiControllerEffectControl1 = 12,
+	MidiControllerEffectControl2 = 13,
+	MidiControllerSustain = 64,
+	MidiControllerPortamento = 65,
+	MidiControllerSostenuto = 66,
+	MidiControllerSoftPedal = 67,
+	MidiControllerLegatoFootswitch = 68,
+};
+
 const int MidiChannelCount = 16;
 const int MidiControllerCount = 128;
 const int MidiProgramCount = 128;
 const int MidiMaxVelocity = 127;
+const int MidiMaxControllerValue = 127;
+const int MidiMaxNote = 127;
 
 const int MidiMaxPanning = 127;
 const int MidiMinPanning = -128;
@@ -148,6 +169,16 @@ struct midiEvent
 		return m_data.m_param[0];
 	}
 
+	inline uint8_t controllerNumber() const
+	{
+		return m_data.m_param[0];
+	}
+
+	inline uint8_t controllerValue() const
+	{
+		return m_data.m_param[1];
+	}
+
 	inline Sint16 velocity() const
 	{
 		return m_data.m_param[1];
