@@ -139,8 +139,16 @@ protected slots:
 
 	void zoomingChanged();
 	void quantizeChanged();
-		
+
+	void updateSemiToneMarkerMenu();
+
 	void changeNoteEditMode( int i );
+	void markSemiTone( int i );
+
+
+signals:
+	void semiToneMarkerMenuScaleSetEnabled(bool);
+	void semiToneMarkerMenuChordSetEnabled(bool);
 
 
 private:
@@ -170,6 +178,14 @@ private:
 		NoteEditCount // make sure this one is always last
 	};
 
+	enum semiToneMarkerAction
+	{
+		stmaUnmarkAll,
+		stmaMarkCurrentSemiTone,
+		stmaMarkCurrentScale,
+		stmaMarkCurrentChord,
+	};
+
 	enum pianoRollKeyTypes
 	{
 		PR_WHITE_KEY_SMALL,
@@ -179,7 +195,9 @@ private:
 
 	QVector<QString> m_nemStr; // gui names of each edit mode
 	QMenu * m_noteEditMenu; // when you right click below the key area
-	QSignalMapper * m_signalMapper; // to keep track of edit mode events
+
+	QList<int> m_markedSemiTones;
+	QMenu * m_semiToneMarkerMenu; // when you right click on the key area
 
 	pianoRoll();
 	pianoRoll( const pianoRoll & );
@@ -204,7 +222,7 @@ private:
 	inline int noteEditRight() const;
 	inline int noteEditLeft() const;
 	
-	void dragNotes( int x, int y, bool alt );
+	void dragNotes( int x, int y, bool alt, bool shift );
 		
 	static const int cm_scrollAmtHoriz = 10;
 	static const int cm_scrollAmtVert = 1;
@@ -240,10 +258,14 @@ private:
 	comboBox * m_zoomingComboBox;
 	comboBox * m_quantizeComboBox;
 	comboBox * m_noteLenComboBox;
+	comboBox * m_scaleComboBox;
+	comboBox * m_chordComboBox;
 
 	ComboBoxModel m_zoomingModel;
 	ComboBoxModel m_quantizeModel;
 	ComboBoxModel m_noteLenModel;
+	ComboBoxModel m_scaleModel;
+	ComboBoxModel m_chordModel;
 
 
 
