@@ -1,7 +1,7 @@
 /*
  * RemoteVstPlugin.cpp - LMMS VST Support Layer (RemotePlugin client)
  *
- * Copyright (c) 2005-2010 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2012 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -163,6 +163,12 @@ public:
 	// determine product-string of plugin
 	const char * pluginProductString();
 
+	// determine name of current program
+	const char * programName();
+
+	// send name of current program back to host
+	void sendCurrentProgramName();
+
 	// do a complete parameter-dump and post it
 	void getParameterDump();
 
@@ -178,6 +184,21 @@ public:
 	// restore settings chunk of plugin from file
 	void loadChunkFromFile( const std::string & _file, int _len );
 
+	// restore settings chunk of plugin from file
+	void loadPresetFile( const std::string & _file );
+
+	// sets given program index
+	void setProgram( int index );
+
+	// rotate current program by given offset
+	void rotateProgram( int offset );
+
+	// Load names of presets/programs
+	void getProgramNames();
+
+	// Save presets/programs
+	void savePreset( const std::string & _file );
+
 	// number of inputs
 	virtual int inputCount() const
 	{
@@ -240,13 +261,23 @@ private:
 		lock();
 		if( m_plugin )
 		{
-			ret = m_plugin->dispatcher( m_plugin, cmd, param1,
-							param2, p, f );
+			ret = m_plugin->dispatcher( m_plugin, cmd, param1, param2, p, f );
 		}
 		unlock();
 		return ret;
 	}
 
+	// thread-safe dispatching of plugin
+	int pluginDispatchNoLocking( int cmd, int param1 = 0, int param2 = 0, void * p = NULL, float f = 0 )
+	{
+		if( m_plugin )
+		{
+			return m_plugin->dispatcher( m_plugin, cmd, param1, param2, p, f );
+		}
+		return 0;
+	}
+
+
 	std::string m_shortName;
 
 	HINSTANCE m_libInst;
@@ -270,6 +301,7 @@ private:
 
 	bpm_t m_bpm;
 	double m_currentSamplePos;
+	int m_currentProgram;
 
 } ;
 
@@ -291,7 +323,8 @@ RemoteVstPlugin::RemoteVstPlugin( key_t _shm_in, key_t _shm_out ) :
 	m_outputs( NULL ),
 	m_midiEvents(),
 	m_bpm( 0 ),
-	m_currentSamplePos( 0 )
+	m_currentSamplePos( 0 ),
+	m_currentProgram( -1 )
 {
 	pthread_mutex_init( &m_pluginLock, NULL );
 
@@ -299,7 +332,7 @@ RemoteVstPlugin::RemoteVstPlugin( key_t _shm_in, key_t _shm_out ) :
 
 	// process until we have loaded the plugin
 	while( 1 )
-        {
+	{
 		message m = receiveMessage();
 		processMessage( m );
 		if( m.id == IdVstLoadPlugin || m.id == IdQuit )
@@ -386,6 +419,54 @@ bool RemoteVstPlugin::processMessage( const message & _m )
 			sendMessage( IdLoadSettingsFromFile );
 			break;
 
+		case IdLoadPresetFile:
+			loadPresetFile( _m.getString( 0 ) );
+			sendMessage( IdLoadPresetFile );
+			break;
+
+		case IdVstSetProgram:
+			setProgram( _m.getInt( 0 ) );
+			sendMessage( IdVstSetProgram );
+			break;
+
+		case IdVstCurrentProgram:
+			sendMessage( message( IdVstCurrentProgram ).addInt( m_currentProgram ) );
+			break;
+
+		case IdVstRotateProgram:
+			rotateProgram( _m.getInt( 0 ) );
+			sendMessage( IdVstRotateProgram );
+			break;
+
+		case IdVstProgramNames:
+			getProgramNames();
+			break;
+
+		case IdSavePresetFile:
+			savePreset( _m.getString( 0 ) );
+			sendMessage( IdSavePresetFile );
+			break;
+
+		case IdVstSetParameter:
+			lock();
+			m_plugin->setParameter( m_plugin, _m.getInt( 0 ), _m.getFloat( 1 ) );
+			unlock();
+			sendMessage( IdVstSetParameter );
+			break;
+
+
+		case IdVstIdleUpdate:
+		{
+			int newCurrentProgram = pluginDispatch( effGetProgram );
+			if( newCurrentProgram != m_currentProgram )
+			{
+				m_currentProgram = newCurrentProgram;
+				sendCurrentProgramName();
+			}
+
+			break;
+		}
+
 		default:
 			return RemotePluginClient::processMessage( _m );
 	}
@@ -567,10 +648,7 @@ bool RemoteVstPlugin::load( const std::string & _plugin_file )
 
 	if( m_plugin->magic != kEffectMagic )
 	{
-		char buf[256];
-		sprintf( buf, "%s is not a VST plugin\n",
-							_plugin_file.c_str() );
-		debugMessage( buf );
+		debugMessage( "File is not a VST plugin\n" );
 		return false;
 	}
 
@@ -633,6 +711,8 @@ void RemoteVstPlugin::process( const sampleFrame * _in, sampleFrame * _out )
 		memset( m_outputs[i], 0, bufferSize() * sizeof( float ) );
 	}
 
+	lock();
+
 #ifdef OLD_VST_SDK
 	if( m_plugin->flags & effFlagsCanReplacing )
 	{
@@ -648,8 +728,9 @@ void RemoteVstPlugin::process( const sampleFrame * _in, sampleFrame * _out )
 	}
 #endif
 
-	m_currentSamplePos += bufferSize();
+	unlock();
 
+	m_currentSamplePos += bufferSize();
 }
 
 
@@ -726,18 +807,52 @@ const char * RemoteVstPlugin::pluginProductString()
 
 
 
+const char * RemoteVstPlugin::programName()
+{
+	static char buf[24];
+
+	memset( buf, 0, sizeof( buf ) );
+
+	pluginDispatch( effGetProgramName, 0, 0, buf );
+
+	buf[23] = 0;
+
+	return buf;
+}
+
+
+
+void RemoteVstPlugin::sendCurrentProgramName()
+{
+	char presName[64];
+	sprintf( presName, "%d/%d: %s", pluginDispatch( effGetProgram ) + 1, m_plugin->numPrograms, programName() );
+
+	sendMessage( message( IdVstCurrentProgramName ).addString( presName ) );
+}
+
+
+
 void RemoteVstPlugin::getParameterDump()
 {
-	VstParameterProperties vst_props;
+	lock();
+
 	message m( IdVstParameterDump );
 	m.addInt( m_plugin->numParams );
+
 	for( int i = 0; i < m_plugin->numParams; ++i )
 	{
-		pluginDispatch( effGetParameterProperties, i, 0, &vst_props );
+		char paramName[32];
+		memset( paramName, 0, sizeof( paramName ) );
+		pluginDispatchNoLocking( effGetParamName, i, 0, paramName );
+		paramName[sizeof(paramName)-1] = 0;
+
 		m.addInt( i );
-		m.addString( vst_props.shortLabel );
+		m.addString( paramName );
 		m.addFloat( m_plugin->getParameter( m_plugin, i ) );
 	}
+
+	unlock();
+
 	sendMessage( m );
 }
 
@@ -746,6 +861,7 @@ void RemoteVstPlugin::getParameterDump()
 
 void RemoteVstPlugin::setParameterDump( const message & _m )
 {
+	lock();
 	const int n = _m.getInt( 0 );
 	const int params = ( n > m_plugin->numParams ) ?
 					m_plugin->numParams : n;
@@ -758,6 +874,7 @@ void RemoteVstPlugin::setParameterDump( const message & _m )
 		item.value = _m.getFloat( ++p );
 		m_plugin->setParameter( m_plugin, item.index, item.value );
 	}
+	unlock();
 }
 
 
@@ -812,6 +929,296 @@ void RemoteVstPlugin::saveChunkToFile( const std::string & _file )
 
 
 
+void RemoteVstPlugin::setProgram( int program )
+{
+	if( isInitialized() == false )
+	{
+		return;
+	}
+
+	if( program < 0 )
+	{
+		program = 0;
+	}
+	else if( program >= m_plugin->numPrograms )
+	{
+		program = m_plugin->numPrograms - 1;
+	}
+	pluginDispatch( effSetProgram, 0, program );
+
+	sendCurrentProgramName();
+}
+
+
+
+
+void RemoteVstPlugin::rotateProgram( int offset )
+{
+	if( isInitialized() == false )
+	{
+		return;
+	}
+
+	int newProgram = pluginDispatch( effGetProgram ) + offset;
+
+	if( newProgram < 0 )
+	{
+		newProgram = 0;
+	}
+	else if( newProgram >= m_plugin->numPrograms )
+	{
+		newProgram = m_plugin->numPrograms - 1;
+	}
+	pluginDispatch( effSetProgram, 0, newProgram );
+
+	sendCurrentProgramName();
+}
+
+
+
+
+void RemoteVstPlugin::getProgramNames()
+{
+	char presName[1024+256*30];
+	char curProgName[30];
+	if (isInitialized() == false) return;
+	bool progNameIndexed = ( pluginDispatch( 29, 0, -1, curProgName ) == 1 );
+
+	if (m_plugin->numPrograms > 1) {
+		if (progNameIndexed) {
+			for (int i = 0; i< (m_plugin->numPrograms >= 256?256:m_plugin->numPrograms); i++)
+			{
+				pluginDispatch( 29, i, -1, curProgName );
+				if (i == 0) 	sprintf( presName, "%s", curProgName );
+				else		sprintf( presName + strlen(presName), "|%s", curProgName );
+			}
+		}
+		else
+		{
+			int currProgram = pluginDispatch( effGetProgram );
+			for (int i = 0; i< (m_plugin->numPrograms >= 256?256:m_plugin->numPrograms); i++)
+			{
+				pluginDispatch( effSetProgram, 0, i );
+				if (i == 0) 	sprintf( presName, "%s", programName() );
+				else		sprintf( presName + strlen(presName), "|%s", programName() );
+			}
+			pluginDispatch( effSetProgram, 0, currProgram );
+		}
+	} else sprintf( presName, "%s", programName() );
+
+	presName[sizeof(presName)-1] = 0;
+
+	sendMessage( message( IdVstProgramNames ).addString( presName ) );
+}
+
+
+
+
+inline unsigned int endian_swap(unsigned int& x)
+{
+    return (x>>24) | ((x<<8) & 0x00FF0000) | ((x>>8) & 0x0000FF00) | (x<<24);
+}
+
+struct sBank
+{
+	unsigned int chunkMagic;
+	unsigned int byteSize;
+	unsigned int fxMagic;
+	unsigned int version;
+	unsigned int fxID;
+	unsigned int fxVersion;
+	unsigned int numPrograms;
+	char prgName[28];
+};
+
+void RemoteVstPlugin::savePreset( const std::string & _file )
+{
+	unsigned int chunk_size = 0;
+	sBank * pBank = ( sBank* ) new char[ sizeof( sBank ) ];
+	char progName[ 128 ] = { 0 };
+	char* data = NULL;
+	const bool chunky = ( m_plugin->flags & ( 1 << 5 ) ) != 0;
+	bool isPreset = _file.substr( _file.find_last_of( "." ) + 1 )  == "fxp";
+	int presNameLen = _file.find_last_of( "/" ) + _file.find_last_of( "\\" ) + 2;
+
+	if (isPreset)
+	{
+		for (size_t i = 0; i < _file.length() - 4 - presNameLen; i++)
+			progName[i] = i < 23 ? _file[presNameLen + i] : 0;
+		pluginDispatch( 4, 0, 0, progName );
+	}
+	if ( chunky )
+		chunk_size = pluginDispatch( 23, isPreset, 0, &data );
+	else {
+		if (isPreset) {
+			chunk_size = m_plugin->numParams * sizeof( float );
+			data = new char[ chunk_size ];
+			unsigned int* toUIntArray = reinterpret_cast<unsigned int*>( data );
+			lock();
+			for ( int i = 0; i < m_plugin->numParams; i++ )
+			{
+				float value = m_plugin->getParameter( m_plugin, i );
+				unsigned int * pValue = ( unsigned int * ) &value;
+				toUIntArray[ i ] = endian_swap( *pValue );
+			}
+			unlock();
+		} else chunk_size = (((m_plugin->numParams * sizeof( float )) + 56)*m_plugin->numPrograms);
+	}
+
+	pBank->chunkMagic = 0x4B6E6343;
+	pBank->byteSize = chunk_size + ( chunky ? sizeof( int ) : 0 ) + 48;
+	if (!isPreset) pBank->byteSize += 100;
+	pBank->byteSize = endian_swap( pBank->byteSize );
+	pBank->fxMagic = chunky ? 0x68435046 : 0x6B437846;
+	if (!isPreset && chunky) pBank->fxMagic = 0x68434246;
+	if (!isPreset &&!chunky) pBank->fxMagic = 0x6B427846;
+
+	pBank->version = 0x01000000;
+	unsigned int uIntToFile = (unsigned int) m_plugin->uniqueID;
+	pBank->fxID = endian_swap( uIntToFile );
+	uIntToFile = (unsigned int) pluginVersion();
+	pBank->fxVersion = endian_swap( uIntToFile );
+	uIntToFile = (unsigned int) chunky ? m_plugin->numPrograms : m_plugin->numParams;
+	if (!isPreset &&!chunky) uIntToFile = (unsigned int) m_plugin->numPrograms;
+	pBank->numPrograms = endian_swap( uIntToFile );
+
+	FILE * stream = fopen( _file.c_str(), "w" );
+	fwrite ( pBank, 1, 28, stream );
+	fwrite ( progName, 1, isPreset ? 28 : 128, stream );
+	if ( chunky ) {
+		uIntToFile = endian_swap( chunk_size );
+		fwrite ( &uIntToFile, 1, 4, stream );
+	}
+	if (pBank->fxMagic != 0x6B427846 )
+		fwrite ( data, 1, chunk_size, stream );
+	else {
+		int numPrograms = m_plugin->numPrograms;
+		int currProgram = pluginDispatch( effGetProgram );
+		chunk_size = (m_plugin->numParams * sizeof( float ));
+		pBank->byteSize = chunk_size + 48;
+		pBank->byteSize = endian_swap( pBank->byteSize );
+		pBank->fxMagic = 0x6B437846;
+		uIntToFile = (unsigned int) m_plugin->numParams;
+		pBank->numPrograms = endian_swap( uIntToFile );
+		data = new char[ chunk_size ];
+		unsigned int* pValue,* toUIntArray = reinterpret_cast<unsigned int*>( data );
+		float value;
+		for (int j = 0; j < numPrograms; j++) {
+			pluginDispatch( effSetProgram, 0, j );
+			pluginDispatch( effGetProgramName, 0, 0, pBank->prgName );
+			fwrite ( pBank, 1, 56, stream );
+			lock();
+			for ( int i = 0; i < m_plugin->numParams; i++ )
+			{
+				value = m_plugin->getParameter( m_plugin, i );
+				pValue = ( unsigned int * ) &value;
+				toUIntArray[ i ] = endian_swap( *pValue );
+			}
+			unlock();
+			fwrite ( data, 1, chunk_size, stream );
+		}
+		pluginDispatch( effSetProgram, 0, currProgram );
+	}
+	fclose( stream );
+
+	if ( !chunky ) 
+		delete[] data;
+	delete[] (sBank*)pBank;
+
+}
+
+
+
+
+void RemoteVstPlugin::loadPresetFile( const std::string & _file )
+{
+	void * chunk = NULL;
+	unsigned int * pLen = new unsigned int[ 1 ];
+	unsigned int len = 0;
+	sBank * pBank = (sBank*) new char[ sizeof( sBank ) ];
+	FILE * stream = fopen( _file.c_str(), "r" );
+	fread ( pBank, 1, 56, stream );
+        pBank->fxID = endian_swap( pBank->fxID );
+	pBank->numPrograms = endian_swap( pBank->numPrograms );
+	unsigned int toUInt;
+	float * pFloat;
+
+	if (m_plugin->uniqueID != pBank->fxID) {
+		sendMessage( message( IdVstCurrentProgramName ).
+					addString( "Error: Plugin UniqID not match" ) );
+		fclose( stream );
+		delete[] (unsigned int*)pLen;
+		delete[] (sBank*)pBank;
+		return;
+	}
+
+	if( _file.substr( _file.find_last_of( "." ) + 1 ) != "fxp" )
+		fseek ( stream , 156 , SEEK_SET );
+
+	if(pBank->fxMagic != 0x6B427846) {
+		if(pBank->fxMagic != 0x6B437846) {
+			fread (pLen, 1, 4, stream);
+			chunk = new char[len = endian_swap(*pLen)];
+		} else chunk = new char[len = sizeof(float)*pBank->numPrograms];
+		fread (chunk, len, 1, stream);
+		fclose( stream );
+	}
+
+	if(_file.substr(_file.find_last_of(".") + 1) == "fxp") {
+		pBank->prgName[23] = 0;
+		pluginDispatch( 4, 0, 0, pBank->prgName );
+		if(pBank->fxMagic != 0x6B437846)
+			pluginDispatch( 24, 1, len, chunk );
+		else
+		{
+			lock();
+			unsigned int* toUIntArray = reinterpret_cast<unsigned int*>( chunk );
+			for (int i = 0; i < pBank->numPrograms; i++ )
+			{
+				toUInt = endian_swap( toUIntArray[ i ] );
+				pFloat = ( float* ) &toUInt;
+				m_plugin->setParameter( m_plugin, i, *pFloat );
+			}
+			unlock();
+		}
+	} else {
+		if(pBank->fxMagic != 0x6B427846) {
+			pluginDispatch( 24, 0, len, chunk );
+		} else {
+			int numPrograms = pBank->numPrograms;
+			unsigned int * toUIntArray;
+			int currProgram = pluginDispatch( effGetProgram );
+			chunk = new char[ len = sizeof(float)*m_plugin->numParams ];
+			toUIntArray = reinterpret_cast<unsigned int *>( chunk );
+			lock();
+			for (int i =0; i < numPrograms; i++) {
+				fread (pBank, 1, 56, stream);
+				fread (chunk, len, 1, stream);
+				pluginDispatch( effSetProgram, 0, i );
+				pBank->prgName[23] = 0;
+				pluginDispatch( 4, 0, 0, pBank->prgName );
+				for (int j = 0; j < m_plugin->numParams; j++ ) {
+					toUInt = endian_swap( toUIntArray[ j ] );
+					pFloat = ( float* ) &toUInt;
+					m_plugin->setParameter( m_plugin, j, *pFloat );
+				}
+			}
+			unlock();
+			pluginDispatch( effSetProgram, 0, currProgram );
+			fclose( stream );
+		}
+	}
+
+	sendCurrentProgramName();
+
+	delete[] (unsigned int*)pLen;
+	delete[] (sBank*)pBank;
+	delete[] (char*)chunk;
+}
+
+
+
+
 void RemoteVstPlugin::loadChunkFromFile( const std::string & _file, int _len )
 {
 	char * buf = NULL;
