@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_PostScript.cxx 7653 2010-06-24 08:55:04Z manolo $"
+// "$Id: Fl_PostScript.cxx 8623 2011-04-24 17:09:41Z AlbrechtS $"
 //
 // PostScript device support for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 2010 by Bill Spitzak and others.
+// Copyright 2010-2011 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -25,15 +25,22 @@
 //     http://www.fltk.org/str.php
 //
 
+#include <config.h>
 #include <FL/Fl.H>
 #include <FL/fl_ask.H>
 #include <FL/fl_draw.H>
 #include <stdio.h>
 #include <FL/Fl_PostScript.H>
 #include <FL/Fl_Native_File_Chooser.H>
+#if defined(USE_X11)
+#include "Fl_Font.H"
+#if USE_XFT
+#include <X11/Xft/Xft.h>
+#endif
+#endif
 
-const char *Fl_PostScript_Graphics_Driver::device_type = "Fl_PostScript_Graphics_Driver";
-const char *Fl_PostScript_File_Device::device_type = "Fl_PostScript_File_Device";
+const char *Fl_PostScript_Graphics_Driver::class_id = "Fl_PostScript_Graphics_Driver";
+const char *Fl_PostScript_File_Device::class_id = "Fl_PostScript_File_Device";
 /** \brief Label of the PostScript file chooser window */
 const char *Fl_PostScript_File_Device::file_chooser_title = "Select a .ps file";
 
@@ -47,7 +54,6 @@ Fl_PostScript_Graphics_Driver::Fl_PostScript_Graphics_Driver(void)
   lang_level_ = 2;
   mask = 0;
   ps_filename_ = NULL;
-  type_ = device_type;
   scale_x = scale_y = 1.;
   bg_r = bg_g = bg_b = 255;
 }
@@ -62,7 +68,6 @@ Fl_PostScript_Graphics_Driver::~Fl_PostScript_Graphics_Driver() {
  */
 Fl_PostScript_File_Device::Fl_PostScript_File_Device(void)
 {
-  type_ = device_type;
 #ifdef __APPLE__
   gc = fl_gc; // the display context is used by fl_text_extents()
 #endif
@@ -85,9 +90,10 @@ Fl_PostScript_Graphics_Driver *Fl_PostScript_File_Device::driver()
  @param pagecount The total number of pages to be created.
  @param format Desired page format.
  @param layout Desired page layout.
- @return 0 iff OK, 1 if user cancelled the file dialog, 2 if fopen failed on user-selected output file.
+ @return 0 if OK, 1 if user cancelled the file dialog, 2 if fopen failed on user-selected output file.
  */
-int Fl_PostScript_File_Device::start_job (int pagecount, enum Fl_PostScript_Graphics_Driver::Page_Format format, enum Fl_PostScript_Graphics_Driver::Page_Layout layout)
+int Fl_PostScript_File_Device::start_job (int pagecount, enum Fl_Paged_Device::Page_Format format, 
+					  enum Fl_Paged_Device::Page_Layout layout)
 {
   Fl_Native_File_Chooser fnfc;
   fnfc.title(Fl_PostScript_File_Device::file_chooser_title);
@@ -105,22 +111,29 @@ int Fl_PostScript_File_Device::start_job (int pagecount, enum Fl_PostScript_Grap
   return 0;
 }
 
+static int dont_close(FILE *f) 
+{
+  return 0;
+}
+
 /**
  @brief Begins the session where all graphics requests will go to FILE pointer.
  *
- @param ps_output A writable FILE pointer that will receive PostScript output and that will be closed
- when end_job() will be called.
+ @param ps_output A writable FILE pointer that will receive PostScript output and that should not be closed
+ until after end_job() has been called.
  @param pagecount The total number of pages to be created.
  @param format Desired page format.
  @param layout Desired page layout.
  @return always 0.
  */
-int Fl_PostScript_File_Device::start_job (FILE *ps_output, int pagecount, enum Fl_PostScript_Graphics_Driver::Page_Format format, enum Fl_PostScript_Graphics_Driver::Page_Layout layout)
+int Fl_PostScript_File_Device::start_job (FILE *ps_output, int pagecount, 
+    enum Fl_Paged_Device::Page_Format format, enum Fl_Paged_Device::Page_Layout layout)
 {
   Fl_PostScript_Graphics_Driver *ps = driver();
   ps->output = ps_output;
   ps->ps_filename_ = NULL;
   ps->start_postscript(pagecount, format, layout);
+  ps->close_command(dont_close); // so that end_job() doesn't close the file
   this->set_current();
   return 0;
 }
@@ -139,46 +152,6 @@ Fl_PostScript_File_Device::~Fl_PostScript_File_Device() {
   #include "print_panel.cxx"
 #endif
 
-const Fl_PostScript_Graphics_Driver::page_format Fl_PostScript_Graphics_Driver::page_formats[NO_PAGE_FORMATS] = { // order of enum Page_Format
-// comes from appendix B of 5003.PPD_Spec_v4.3.pdf
-
-  // A* // index(Ai) = i
-  {2384, 3370, "A0"},
-  {1684, 2384, "A1"},
-  {1191, 1684, "A2"},
-  { 842, 1191, "A3"},
-  { 595,  842, "A4"},
-  { 420,  595, "A5"},
-  { 297,  420, "A6"},
-  { 210,  297, "A7"},
-  { 148,  210, "A8"},
-  { 105,  148, "A9"},
-
-  // B* // index(Bi) = i+10
-  {2920, 4127, "B0"},
-  {2064, 2920, "B1"},
-  {1460, 2064, "B2"},
-  {1032, 1460, "B3"},
-  { 729, 1032, "B4"},
-  { 516,  729, "B5"},
-  { 363,  516, "B6"},
-  { 258,  363, "B7"},
-  { 181,  258, "B8"},
-  { 127,  181, "B9"},
-  {  91,  127, "B10"},
-
-  // others
-  { 459,  649, "EnvC5"}, // envelope
-  { 312,  624, "EnvDL"}, // envelope
-  { 522,  756, "Executive"},
-  { 595,  935, "Folio"},
-  {1224,  792, "Ledger"}, // landscape
-  { 612, 1008, "Legal"},
-  { 612,  792, "Letter"},
-  { 792, 1224, "Tabloid"},
-  { 297,  684, "Env10"} // envelope
-};
-
 //  Prolog string 
 
 static const char * prolog =
@@ -270,7 +243,7 @@ static const char * prolog =
 
 "/MI { GS /py exch def /px exch def /sy exch def /sx exch def \n"
 "translate \n"
-"sx sy scale px py false \n"
+"sx sy scale px py true \n"
 "[ px 0 0 py neg 0 py ]\n"
 "currentfile /ASCIIHexDecode filter\n"
 "imagemask GR\n"
@@ -300,7 +273,8 @@ static const char * prolog =
 // show at position with desired width
 // usage:
 // width (string) x y show_pos_width
-"/show_pos_width {GS moveto dup dup stringwidth pop exch length exch 3 index exch sub exch "
+"/show_pos_width {GS moveto dup dup stringwidth pop exch length 2 div dup 2 le {pop 9999} if "
+"1 sub exch 3 index exch sub exch "
 "div 0 2 index 1 -1 scale ashow pop pop GR} bind def\n" // spacing altered to match desired width
 //"/show_pos_width {GS moveto dup stringwidth pop 3 2 roll exch div -1 matrix scale concat "
 //"show GR } bind def\n" // horizontally scaled text to match desired width
@@ -330,8 +304,6 @@ static const char * prolog_2 =  // prolog relevant only if lang_level >1
 "IDD image GR} bind def\n"
 
 // gray image dict 
-
-
 "/GII {GS /inter exch def /py exch def /px exch def /sy exch def /sx exch def \n"
 "translate \n"
 "sx sy scale\n"
@@ -351,28 +323,90 @@ static const char * prolog_2 =  // prolog relevant only if lang_level >1
 "end\n"
 "IDD image GR} bind def\n"
 
-// procedure to modify a font to use ISOLatin1 encoding (iso-8859-1)
-// and to keep its name unchanged
-"/ToLatin1 { dup findfont dup length dict "
-"begin {def} forall /Encoding ISOLatin1Encoding def currentdict end definefont pop } def\n"
-// modify all fonts to use ISOLatin1 encoding
-"/Helvetica ToLatin1 "
-"/Helvetica-Bold ToLatin1 "
-"/Helvetica-Oblique ToLatin1 "
-"/Helvetica-BoldOblique ToLatin1 \n"
-"/Courier ToLatin1 "
-"/Courier-Bold ToLatin1 "
-"/Courier-Oblique ToLatin1 "
-"/Courier-BoldOblique ToLatin1 \n"
-"/Times ToLatin1 "
-"/Times-Bold ToLatin1 "
-"/Times-Italic ToLatin1 "
-"/Times-BoldItalic ToLatin1 \n"
+// Create a custom PostScript font derived from PostScript standard text fonts
+// The encoding of this custom font is as follows:
+// 0000-00FF  coincides with Unicode, that is to ASCII + Latin-1
+// 0100-017F  coincides with Unicode, that is to Latin Extended-A
+// 0180-01A6  encodes miscellaneous characters present in PostScript standard text fonts
+
+// use ISOLatin1Encoding for all text fonts
+"/ToISO { dup findfont dup length dict copy begin /Encoding ISOLatin1Encoding def currentdict end definefont pop } def\n"
+"/Helvetica ToISO /Helvetica-Bold ToISO /Helvetica-Oblique ToISO /Helvetica-BoldOblique ToISO \n"
+"/Courier ToISO /Courier-Bold ToISO /Courier-Oblique ToISO /Courier-BoldOblique ToISO \n"
+"/Times-Roman ToISO /Times-Bold ToISO /Times-Italic ToISO /Times-BoldItalic ToISO \n"
+
+// define LatinExtA, the encoding of Latin-extended-A + some additional characters
+// see http://www.adobe.com/devnet/opentype/archives/glyphlist.txt for their names
+"/LatinExtA \n"
+"[ "
+" /Amacron /amacron /Abreve /abreve /Aogonek /aogonek\n" // begin of Latin Extended-A code page
+" /Cacute  /cacute  /Ccircumflex  /ccircumflex  /Cdotaccent  /cdotaccent  /Ccaron  /ccaron \n"
+" /Dcaron  /dcaron   /Dcroat  /dcroat\n"
+" /Emacron  /emacron  /Ebreve  /ebreve  /Edotaccent  /edotaccent  /Eogonek  /eogonek  /Ecaron  /ecaron\n"
+" /Gcircumflex  /gcircumflex  /Gbreve  /gbreve  /Gdotaccent  /gdotaccent  /Gcommaaccent  /gcommaaccent \n"
+" /Hcircumflex /hcircumflex  /Hbar  /hbar  \n"
+" /Itilde  /itilde  /Imacron  /imacron  /Ibreve  /ibreve  /Iogonek  /iogonek /Idotaccent  /dotlessi  \n"
+" /IJ  /ij  /Jcircumflex  /jcircumflex\n"
+" /Kcommaaccent  /kcommaaccent  /kgreenlandic  \n"
+" /Lacute  /lacute  /Lcommaaccent  /lcommaaccent   /Lcaron  /lcaron  /Ldotaccent /ldotaccent   /Lslash  /lslash \n"
+" /Nacute  /nacute  /Ncommaaccent  /ncommaaccent  /Ncaron  /ncaron  /napostrophe  /Eng  /eng  \n"
+" /Omacron  /omacron /Obreve  /obreve  /Ohungarumlaut  /ohungarumlaut  /OE  /oe \n"
+" /Racute  /racute  /Rcommaaccent  /rcommaaccent  /Rcaron  /rcaron \n"
+" /Sacute /sacute  /Scircumflex  /scircumflex  /Scedilla /scedilla /Scaron  /scaron \n"
+" /Tcommaaccent  /tcommaaccent  /Tcaron  /tcaron  /Tbar  /tbar \n"
+" /Utilde  /utilde /Umacron /umacron  /Ubreve  /ubreve  /Uring  /uring  /Uhungarumlaut  /uhungarumlaut  /Uogonek /uogonek \n"
+" /Wcircumflex  /wcircumflex  /Ycircumflex  /ycircumflex  /Ydieresis \n"
+" /Zacute /zacute /Zdotaccent /zdotaccent /Zcaron /zcaron \n"
+" /longs \n" // end of Latin Extended-A code page
+" /florin  /circumflex  /caron  /breve  /dotaccent  /ring \n" // remaining characters from PostScript standard text fonts
+" /ogonek  /tilde  /hungarumlaut  /endash /emdash \n"
+" /quoteleft  /quoteright  /quotesinglbase  /quotedblleft  /quotedblright \n"
+" /quotedblbase  /dagger  /daggerdbl  /bullet  /ellipsis \n"
+" /perthousand  /guilsinglleft  /guilsinglright  /fraction  /Euro \n"
+" /trademark /partialdiff  /Delta /summation  /radical \n"
+" /infinity /notequal /lessequal /greaterequal /lozenge \n"
+" /fi /fl /apple \n"
+" ] def \n"
+// deal with alternative PostScript names of some characters
+" /mycharstrings /Helvetica findfont /CharStrings get def\n"
+" /PSname2 { dup mycharstrings exch known {LatinExtA 3 -1 roll 3 -1 roll put}{pop pop} ifelse } def \n"
+" 16#20 /Gdot PSname2 16#21 /gdot PSname2 16#30 /Idot PSname2 16#3F /Ldot PSname2 16#40 /ldot PSname2 16#7F /slong PSname2 \n"
+
+// proc that gives LatinExtA encoding to a font
+"/ToLatinExtA { findfont dup length dict copy begin /Encoding LatinExtA def currentdict end definefont pop } def\n"
+// create Ext-versions of standard fonts that use LatinExtA encoding \n"
+"/HelveticaExt /Helvetica ToLatinExtA \n"
+"/Helvetica-BoldExt /Helvetica-Bold ToLatinExtA /Helvetica-ObliqueExt /Helvetica-Oblique ToLatinExtA  \n"
+"/Helvetica-BoldObliqueExt /Helvetica-BoldOblique ToLatinExtA  \n"
+"/CourierExt /Courier ToLatinExtA /Courier-BoldExt /Courier-Bold ToLatinExtA  \n"
+"/Courier-ObliqueExt /Courier-Oblique ToLatinExtA /Courier-BoldObliqueExt /Courier-BoldOblique ToLatinExtA \n"
+"/Times-RomanExt /Times-Roman ToLatinExtA /Times-BoldExt /Times-Bold ToLatinExtA  \n"
+"/Times-ItalicExt /Times-Italic ToLatinExtA /Times-BoldItalicExt /Times-BoldItalic ToLatinExtA \n"
+
+// proc to create a Type 0 font with 2-byte encoding 
+// that merges a text font with ISO encoding + same font with LatinExtA encoding
+"/To2byte { 6 dict begin /FontType 0 def \n"
+"/FDepVector 3 1 roll findfont exch findfont 2 array astore def \n"
+"/FontMatrix [1  0  0  1  0  0] def /FMapType 6 def /Encoding [ 0 1 0 ] def\n"
+// 100: Hexa count of ISO array; A7: hexa count of LatinExtA array
+"/SubsVector < 01 0100 00A7 > def\n" 
+"currentdict end definefont pop } def\n"
+// create Type 0 versions of standard fonts
+"/Helvetica2B /HelveticaExt /Helvetica To2byte \n"
+"/Helvetica-Bold2B /Helvetica-BoldExt /Helvetica-Bold To2byte \n"
+"/Helvetica-Oblique2B /Helvetica-ObliqueExt /Helvetica-Oblique To2byte \n"
+"/Helvetica-BoldOblique2B /Helvetica-BoldObliqueExt /Helvetica-BoldOblique To2byte \n"
+"/Courier2B /CourierExt /Courier To2byte \n"
+"/Courier-Bold2B /Courier-BoldExt /Courier-Bold To2byte \n"
+"/Courier-Oblique2B /Courier-ObliqueExt /Courier-Oblique To2byte \n"
+"/Courier-BoldOblique2B /Courier-BoldObliqueExt /Courier-BoldOblique To2byte \n"
+"/Times-Roman2B /Times-RomanExt /Times-Roman To2byte \n"
+"/Times-Bold2B /Times-BoldExt /Times-Bold To2byte \n"
+"/Times-Italic2B /Times-ItalicExt /Times-Italic To2byte \n"
+"/Times-BoldItalic2B /Times-BoldItalicExt /Times-BoldItalic To2byte \n"
 ;
 
-static const char * prolog_2_pixmap =  // prolog relevant only if lang_level == 2 for pixmaps
-"/pixmap_size { /pixmap_sy exch def /pixmap_sx exch def } bind def\n"
-
+static const char * prolog_2_pixmap =  // prolog relevant only if lang_level == 2 for pixmaps/masked color images
 "/pixmap_mat {[ pixmap_sx 0 0 pixmap_sy neg 0 pixmap_sy ]}  bind def\n"
 
 "/pixmap_dict {"
@@ -384,10 +418,10 @@ static const char * prolog_2_pixmap =  // prolog relevant only if lang_level ==
 "/YStep pixmap_sy\n"
 "/PaintProc "
 "{ begin "
-"pixmap_sx pixmap_sy scale "
+"pixmap_w pixmap_h scale "
 "pixmap_sx pixmap_sy 8 "
 "pixmap_mat "
-"pixmap_data "
+"currentfile /ASCIIHexDecode filter "
 "false 3 "
 "colorimage "
 "end "
@@ -397,28 +431,20 @@ static const char * prolog_2_pixmap =  // prolog relevant only if lang_level ==
 
 "/pixmap_plot {"
 "GS "
-"/pixmap_y exch def /pixmap_x exch def\n"
-"pixmap_x pixmap_y translate\n"
+"/pixmap_sy exch def /pixmap_sx exch def\n"
+"/pixmap_h exch def /pixmap_w exch def\n"
+"translate\n"
 "pixmap_dict matrix makepattern setpattern\n"
-"pixmap_sx pixmap_sy scale\n"
+"pixmap_w pixmap_h scale\n"
 "pixmap_sx pixmap_sy\n"
 "true\n"
 "pixmap_mat\n"
-"pixmap_mask\n"
+"currentfile /ASCIIHexDecode filter\n"
 "imagemask\n"
 "GR\n"
 "} bind def\n"
-
-"/pixmap_loaddata { /pixmap_data currentfile pixmap_sx pixmap_sy 3 mul mul string readhexstring "
-"} bind def\n"
-
-"/pixmap_loadmask { "
-"/pixmap_mask currentfile  pixmap_sx 8 div ceiling cvi pixmap_sy mul  string readhexstring "
-"} bind def\n"
 ;
 
-
-
 static const char * prolog_3 = // prolog relevant only if lang_level >2
 
 // masked color images 
@@ -508,11 +534,12 @@ static const char * prolog_3 = // prolog relevant only if lang_level >2
 
 // end prolog 
 
-int Fl_PostScript_Graphics_Driver::start_postscript (int pagecount, enum Fl_PostScript_Graphics_Driver::Page_Format format, enum Fl_PostScript_Graphics_Driver::Page_Layout layout)
+int Fl_PostScript_Graphics_Driver::start_postscript (int pagecount, 
+    enum Fl_Paged_Device::Page_Format format, enum Fl_Paged_Device::Page_Layout layout)
 //returns 0 iff OK
 {
   int w, h, x;
-  if (format == A4) {
+  if (format == Fl_Paged_Device::A4) {
     left_margin = 18;
     top_margin = 18;
   }
@@ -520,7 +547,7 @@ int Fl_PostScript_Graphics_Driver::start_postscript (int pagecount, enum Fl_Post
     left_margin = 12;
     top_margin = 12;
   }
-  page_format_ = (enum Page_Format)(format | layout);
+  page_format_ = (enum Fl_Paged_Device::Page_Format)(format | layout);
   
   fputs("%!PS-Adobe-3.0\n", output);
   fputs("%%Creator: FLTK\n", output);
@@ -530,10 +557,10 @@ int Fl_PostScript_Graphics_Driver::start_postscript (int pagecount, enum Fl_Post
     fprintf(output, "%%%%Pages: %i\n", pagecount);
   else
     fputs("%%Pages: (atend)\n", output);
-  fprintf(output, "%%%%BeginFeature: *PageSize %s\n", page_formats[format].name );
-  w = page_formats[format].width;
-  h = page_formats[format].height;
-  if (lang_level_ == 3 && (layout & LANDSCAPE) ) { x = w; w = h; h = x; }
+  fprintf(output, "%%%%BeginFeature: *PageSize %s\n", Fl_Paged_Device::page_formats[format].name );
+  w = Fl_Paged_Device::page_formats[format].width;
+  h = Fl_Paged_Device::page_formats[format].height;
+  if (lang_level_ == 3 && (layout & Fl_Paged_Device::LANDSCAPE) ) { x = w; w = h; h = x; }
   fprintf(output, "<</PageSize[%d %d]>>setpagedevice\n", w, h );
   fputs("%%EndFeature\n", output);
   fputs("%%EndComments\n", output);
@@ -568,15 +595,14 @@ int Fl_PostScript_Graphics_Driver::start_postscript (int pagecount, enum Fl_Post
 void Fl_PostScript_Graphics_Driver::recover(){
   color(cr_,cg_,cb_);
   line_style(linestyle_,linewidth_,linedash_);
-  font(font_,size_);
+  font(Fl_Graphics_Driver::font(), Fl_Graphics_Driver::size());
 }
 
 void Fl_PostScript_Graphics_Driver::reset(){
   gap_=1;
   clip_=0;
   cr_=cg_=cb_=0;
-  font_=FL_HELVETICA;
-  size_=12;
+  Fl_Graphics_Driver::font(FL_HELVETICA, 12);
   linewidth_=0;
   linestyle_=FL_SOLID;
   strcpy(linedash_,"");
@@ -614,8 +640,8 @@ void Fl_PostScript_Graphics_Driver::page(double pw, double ph, int media) {
   }
   
   fprintf(output, "%%%%BeginPageSetup\n");
-  if((media & MEDIA) &&(lang_level_>1)){
-    int r = media & REVERSED;
+  if((media & Fl_Paged_Device::MEDIA) &&(lang_level_>1)){
+    int r = media & Fl_Paged_Device::REVERSED;
     if(r) r = 2;
     fprintf(output, "<< /PageSize [%i %i] /Orientation %i>> setpagedevice\n", (int)(pw+.5), (int)(ph+.5), r);
   }
@@ -632,9 +658,9 @@ void Fl_PostScript_Graphics_Driver::page(double pw, double ph, int media) {
   line_style(0);
   fprintf(output, "GS\n");
   
-  if (!((media & MEDIA) &&(lang_level_>1))){
+  if (!((media & Fl_Paged_Device::MEDIA) &&(lang_level_>1))){
     if (pw > ph) {
-      if(media & REVERSED) {
+      if(media & Fl_Paged_Device::REVERSED) {
         fprintf(output, "-90 rotate %i 0 translate\n", int(-pw));
 	}
       else {
@@ -642,25 +668,25 @@ void Fl_PostScript_Graphics_Driver::page(double pw, double ph, int media) {
 	}
       }
       else {
-	if(media & REVERSED)
+	if(media & Fl_Paged_Device::REVERSED)
 	  fprintf(output, "180 rotate %i %i translate\n", int(-pw), int(-ph));
 	}
   }
   fprintf(output, "GS\nCS\n");
-};
+}
 
 void Fl_PostScript_Graphics_Driver::page(int format){
   
   
-  if(format &  LANDSCAPE){
-    ph_=Fl_PostScript_Graphics_Driver::page_formats[format & 0xFF].width;
-    pw_=Fl_PostScript_Graphics_Driver::page_formats[format & 0xFF].height;
+  if(format &  Fl_Paged_Device::LANDSCAPE){
+    ph_=Fl_Paged_Device::page_formats[format & 0xFF].width;
+    pw_=Fl_Paged_Device::page_formats[format & 0xFF].height;
   }else{
-    pw_=Fl_PostScript_Graphics_Driver::page_formats[format & 0xFF].width;
-    ph_=Fl_PostScript_Graphics_Driver::page_formats[format & 0xFF].height;
+    pw_=Fl_Paged_Device::page_formats[format & 0xFF].width;
+    ph_=Fl_Paged_Device::page_formats[format & 0xFF].height;
   }
   page(pw_,ph_,format & 0xFF00);//,orientation only;
-};
+}
 
 void Fl_PostScript_Graphics_Driver::rect(int x, int y, int w, int h) {
   // Commented code does not work, i can't find the bug ;-(
@@ -706,7 +732,7 @@ void Fl_PostScript_Graphics_Driver::xyline(int x, int y, int x1, int y2, int x3)
   fprintf(output,"%i %i LT\n", x3 , y2);
   fprintf(output, "ELP\n");
   fprintf(output, "GR\n");
-};
+}
 
 
 void Fl_PostScript_Graphics_Driver::xyline(int x, int y, int x1, int y2){
@@ -718,7 +744,7 @@ void Fl_PostScript_Graphics_Driver::xyline(int x, int y, int x1, int y2){
   fprintf(output, "%i %i LT\n", x1 , y2 );
   fprintf(output, "ELP\n");
   fprintf(output, "GR\n");
-};
+}
 
 void Fl_PostScript_Graphics_Driver::xyline(int x, int y, int x1){
   fprintf(output, "GS\n");
@@ -728,7 +754,7 @@ void Fl_PostScript_Graphics_Driver::xyline(int x, int y, int x1){
   fprintf(output, "ELP\n");
   
   fprintf(output, "GR\n");
-};
+}
 
 void Fl_PostScript_Graphics_Driver::yxline(int x, int y, int y1, int x2, int y3){
   fprintf(output, "GS\n");
@@ -740,7 +766,7 @@ void Fl_PostScript_Graphics_Driver::yxline(int x, int y, int y1, int x2, int y3)
   fprintf(output , "%i %i LT\n", x2 , y3);
   fprintf(output, "ELP\n");
   fprintf(output, "GR\n");
-};
+}
 
 void Fl_PostScript_Graphics_Driver::yxline(int x, int y, int y1, int x2){
   fprintf(output, "GS\n");
@@ -750,7 +776,7 @@ void Fl_PostScript_Graphics_Driver::yxline(int x, int y, int y1, int x2){
   fprintf(output, "%i %i LT\n", x2 , y1);
   fprintf(output, "ELP\n");
   fprintf(output, "GR\n");
-};
+}
 
 void Fl_PostScript_Graphics_Driver::yxline(int x, int y, int y1){
   fprintf(output, "GS\n");
@@ -759,7 +785,7 @@ void Fl_PostScript_Graphics_Driver::yxline(int x, int y, int y1){
   fprintf(output, "%i %i LT\n", x , y1);
   fprintf(output, "ELP\n");
   fprintf(output, "GR\n");
-};
+}
 
 void Fl_PostScript_Graphics_Driver::loop(int x0, int y0, int x1, int y1, int x2, int y2) {
   fprintf(output, "GS\n");
@@ -841,7 +867,7 @@ void Fl_PostScript_Graphics_Driver::line_style(int style, int width, char* dashe
     linedash_[0]=0;
   char width0 = 0;
   if(!width){
-    width=1; //for screen drawing compatability
+    width=1; //for screen drawing compatibility
     width0=1;
   }
   
@@ -884,57 +910,83 @@ void Fl_PostScript_Graphics_Driver::line_style(int style, int width, char* dashe
     }
   }
   fprintf(output, "] 0 setdash\n");
-};
+}
 
 static const char *_fontNames[] = {
-"Helvetica",
-"Helvetica-Bold",
-"Helvetica-Oblique",
-"Helvetica-BoldOblique",
-"Courier",
-"Courier-Bold",
-"Courier-Oblique",
-"Courier-BoldOblique",
-"Times",
-"Times-Bold",
-"Times-Italic",
-"Times-BoldItalic",
+"Helvetica2B", 
+"Helvetica-Bold2B",
+"Helvetica-Oblique2B",
+"Helvetica-BoldOblique2B",
+"Courier2B",
+"Courier-Bold2B",
+"Courier-Oblique2B",
+"Courier-BoldOblique2B",
+"Times-Roman2B",
+"Times-Bold2B",
+"Times-Italic2B",
+"Times-BoldItalic2B",
 "Symbol",
-"Courier",
-"Courier-Bold",
+"Courier2B",
+"Courier-Bold2B",
 "ZapfDingbats"
 };
 
 void Fl_PostScript_Graphics_Driver::font(int f, int s) {
-  if (f >= FL_FREE_FONT)
-    f = FL_COURIER;
-  fprintf(output, "/%s SF\n" , _fontNames[f]);
-  fprintf(output,"%i FS\n", s);
-  Fl_Display_Device::display_device()->driver()->font(f,s); // Use display fonts for font measurement
-  font_ = f; size_ = s;
-};
-
-void Fl_PostScript_Graphics_Driver::color(Fl_Color c) {
-  //colored_=1;
-  color_=c;
-  Fl::get_color(c, cr_, cg_, cb_);
-  if (cr_==cg_ && cg_==cb_) {
-    double gray = cr_/255.0;
-    fprintf(output, "%g GL\n", gray);
-    
-  } else {
-    double fr, fg, fb;
-    fr = cr_/255.0;
-    fg = cg_/255.0;
-    fb = cb_/255.0;
-    fprintf(output,"%g %g %g SRGB\n", fr , fg , fb);
+  Fl_Graphics_Driver *driver = Fl_Display_Device::display_device()->driver();
+  driver->font(f,s); // Use display fonts for font measurement
+  Fl_Graphics_Driver::font(f, s);
+  Fl_Font_Descriptor *desc = driver->font_descriptor();
+  this->font_descriptor(desc);
+  if (f < FL_FREE_FONT) {
+    float ps_size = s;
+    fprintf(output, "/%s SF\n" , _fontNames[f]);
+#if defined(USE_X11) 
+#if USE_XFT
+    // Xft font height is sometimes larger than the required size (see STR 2566).
+    // Increase the PostScript font size by 15% without exceeding the display font height 
+    int max = desc->font->height;
+    ps_size = s * 1.15;
+    if (ps_size > max) ps_size = max;
+#else
+    // Non-Xft fonts can be smaller than required.
+    // Set the PostScript font size to the display font height 
+    char *name = desc->font->font_name_list[0];
+    char *p = strstr(name, "--");
+    if (p) {
+      sscanf(p + 2, "%f", &ps_size);
+    }
+#endif // USE_XFT
+#endif // USE_X11
+    fprintf(output,"%.1f FS\n", ps_size);
   }
 }
 
+double Fl_PostScript_Graphics_Driver::width(const char *s, int n) {
+  return Fl_Display_Device::display_device()->driver()->width(s, n);
+}
+
+int Fl_PostScript_Graphics_Driver::height() {
+  return Fl_Display_Device::display_device()->driver()->height();
+}
+
+int Fl_PostScript_Graphics_Driver::descent() {
+  return Fl_Display_Device::display_device()->driver()->descent();
+}
+
+void Fl_PostScript_Graphics_Driver::text_extents(const char *c, int n, int &dx, int &dy, int &w, int &h) {
+  Fl_Display_Device::display_device()->driver()->text_extents(c, n, dx, dy, w, h);
+}
+
+
+void Fl_PostScript_Graphics_Driver::color(Fl_Color c) {
+  Fl::get_color(c, cr_, cg_, cb_);
+  color(cr_, cg_, cb_);
+}
+
 void Fl_PostScript_Graphics_Driver::color(unsigned char r, unsigned char g, unsigned char b) {
-  //colored_=1;
-  cr_=r;cg_=g;cb_=b;
-  if (r==g && g==b) {
+  Fl_Graphics_Driver::color( fl_rgb_color(r, g, b) );
+  cr_ = r; cg_ = g; cb_ = b;
+  if (r == g && g == b) {
     double gray = r/255.0;
     fprintf(output, "%g GL\n", gray);
   } else {
@@ -953,32 +1005,162 @@ void Fl_PostScript_Graphics_Driver::draw(int angle, const char *str, int n, int
   fprintf(output, "GR\n");
 }
 
-// outputs in PostScript a UTF8 string replacing non-Latin1 characters by ?
-// and using the same width in points as on display
-void Fl_PostScript_Graphics_Driver::transformed_draw(const char* str, int n, double x, double y) {
-  int len;
-  if (!n || !str || !*str) return;
-  const char *last = str + n;
-  // compute display width of string
-  fprintf(output,"%g (", fl_width(str, n));
-  while (str < last) {
-    // Extract each unicode character of string.
-    // Until 0xFF, UTF codes coincide with iso-Latin1 (iso-8859-1)
-    unsigned utf = fl_utf8decode(str, last, &len);
-    str += len;
-    if (utf > 0xFF) {
-      utf = '?'; // replace non Latin-1 unicodes by ?
+
+// computes the mask for the RGB image img of all pixels with color != bg
+static uchar *calc_mask(uchar *img, int w, int h, Fl_Color bg)
+{
+  uchar red, green, blue, r, g, b;
+  uchar bit, byte, *q;
+  Fl::get_color(bg, red, green, blue);
+  int W = (w+7)/8; // width of mask
+  uchar* mask = new uchar[W * h];
+  q = mask;
+  while (h-- > 0) { // for each row
+    bit = 0x80; // byte with last bit set
+    byte = 0; // next mask byte to compute
+    for (int j = 0; j < w; j++) { // for each column
+      r = *img++; // the pixel color components
+      g = *img++;
+      b = *img++;
+      // if pixel doesn't have bg color, put it in mask
+      if (r != red || g != green || b != blue) byte |= bit;
+      bit = bit>>1; // shift bit one step to the right
+      if (bit == 0) { // single set bit has fallen out
+	*q++ = byte; // enter byte in mask
+	byte = 0; // reset next mask byte to zero
+	bit = 0x80; // and this byte
+	}
       }
-    else if (utf == '(' || utf == ')' || utf == '\\') {
-      putc('\\' , output); // these chars need be escaped
-     }
-    putc(utf, output); // output the latin character
-  }
-  fprintf(output, ") %g %g show_pos_width\n", x, y);
+    if (bit != 0x80) *q++ = byte; // enter last columns' byte in mask
+    }
+  return mask;
 }
 
-struct matrix {double a, b, c, d, x, y;};
-extern matrix * fl_matrix;
+// write to PostScript a bitmap image of a UTF8 string
+static void transformed_draw_extra(const char* str, int n, double x, double y, int w, 
+      FILE *output, Fl_Graphics_Driver *driver, bool rtl) {
+  // scale for bitmask computation
+#if defined(USE_X11) && !USE_XFT
+  float scale = 1; // don't scale because we can't expect to have scalable fonts
+#else
+  float scale = 2;
+#endif
+  Fl_Fontsize old_size = driver->size();
+  Fl_Font fontnum = driver->font();
+  int w_scaled =  (int)(w * (scale + 0.5));
+  int h = (int)(driver->height() * scale);
+  // create an offscreen image of the string
+  Fl_Color text_color = driver->color();
+  Fl_Color bg_color = fl_contrast(FL_WHITE, text_color);
+  Fl_Offscreen off = fl_create_offscreen(w_scaled, (int)(h+3*scale) );
+  fl_begin_offscreen(off);
+  fl_color(bg_color);
+  // color offscreen background with a shade contrasting with the text color
+  fl_rectf(0, 0, w_scaled, (int)(h+3*scale) );
+  fl_color(text_color);
+#if defined(USE_X11) && !USE_XFT
+  // force seeing this font as new so it's applied to the offscreen graphics context
+  fl_graphics_driver->font_descriptor(NULL);
+  fl_font(fontnum, 0);
+#endif
+  fl_font(fontnum, (Fl_Fontsize)(scale * old_size) );
+  int w2 = (int)fl_width(str, n);
+  // draw string in offscreen
+  if (rtl) fl_rtl_draw(str, n, w2, (int)(h * 0.8) );
+  else fl_draw(str, n, 1, (int)(h * 0.8) );
+  // read (most of) the offscreen image
+  uchar *img = fl_read_image(NULL, 1, 1, w2, h, 0);
+  fl_end_offscreen();
+  driver->font(fontnum, old_size);
+  fl_delete_offscreen(off);
+  // compute the mask of what is not the background
+  uchar *mask = calc_mask(img, w2, h, bg_color);
+  delete[] img;
+  // write the string image to PostScript as a scaled bitmask
+  scale = w2 / float(w);
+  fprintf(output, "%g %g %g %g %d %d MI\n", x, y - h*0.77/scale, w2/scale, h/scale, w2, h);
+  uchar *di;
+  int wmask = (w2+7)/8;
+  for (int j = h - 1; j >= 0; j--){
+    di = mask + j * wmask;
+    for (int i = 0; i < wmask; i++){
+      //if (!(i%80)) fprintf(output, "\n"); // don't have lines longer than 255 chars
+      fprintf(output, "%2.2x", *di );
+      di++;
+    }
+    fprintf(output,"\n");
+  }
+  fprintf(output,">\n");
+  delete[] mask;
+}
+
+static int is_in_table(unsigned utf) {
+  unsigned i;
+  static unsigned extra_table_roman[] = { // unicodes/*names*/ of other characters from PostScript standard fonts
+    0x192/*florin*/, 0x2C6/*circumflex*/, 0x2C7/*caron*/, 
+    0x2D8/*breve*/, 0x2D9/*dotaccent*/, 0x2DA/*ring*/, 0x2DB/*ogonek*/, 0x2DC/*tilde*/, 0x2DD/*hungarumlaut*/,
+    0x2013/*endash*/, 0x2014/*emdash*/, 0x2018/*quoteleft*/, 0x2019/*quoteright*/, 
+    0x201A/*quotesinglbase*/, 0x201C/*quotedblleft*/, 0x201D/*quotedblright*/, 0x201E/*quotedblbase*/, 
+    0x2020/*dagger*/, 0x2021/*daggerdbl*/, 0x2022/*bullet*/,
+    0x2026/*ellipsis*/, 0x2030/*perthousand*/, 0x2039/*guilsinglleft*/, 0x203A/*guilsinglright*/, 
+    0x2044/*fraction*/, 0x20AC/*Euro*/, 0x2122/*trademark*/, 
+    0x2202/*partialdiff*/, 0x2206/*Delta*/, 0x2211/*summation*/, 0x221A/*radical*/,
+    0x221E/*infinity*/, 0x2260/*notequal*/, 0x2264/*lessequal*/, 
+    0x2265/*greaterequal*/, 
+    0x25CA/*lozenge*/, 0xFB01/*fi*/, 0xFB02/*fl*/,
+    0xF8FF/*apple*/
+  };
+  for ( i = 0; i < sizeof(extra_table_roman)/sizeof(int); i++) {
+    if (extra_table_roman[i] == utf) return i + 0x180;
+  }
+  return 0;
+}
+
+// outputs in PostScript a UTF8 string using the same width in points as on display
+void Fl_PostScript_Graphics_Driver::transformed_draw(const char* str, int n, double x, double y) {
+  int len, code;
+  if (!n || !str || !*str) return;
+  // compute display width of string
+  int w = (int)width(str, n);
+  if (w == 0) return;
+  if (Fl_Graphics_Driver::font() >= FL_FREE_FONT) {
+    transformed_draw_extra(str, n, x, y, w, output, this, false);
+    return;
+    }
+  fprintf(output, "%d <", w);
+  // transforms UTF8 encoding to our custom PostScript encoding as follows:
+  // extract each unicode character
+  // if unicode <= 0x17F, unicode and PostScript codes are identical
+  // if unicode is one of the values listed in extra_table_roman above
+  //    its PostScript code is 0x180 + the character's rank in extra_table_roman
+  // if unicode is something else, draw all string as bitmap image
+
+  const char *last = str + n;
+  const char *str2 = str;
+  while (str2 < last) {
+    // Extract each unicode character of string.
+    unsigned utf = fl_utf8decode(str2, last, &len);
+    str2 += len;
+    if (utf <= 0x17F) { // until Latin Extended-A
+      ;
+      }
+    else if ( (code = is_in_table(utf)) != 0) { // other handled characters
+      utf = code;
+      }
+    else { // unhandled character: draw all string as bitmap image
+      fprintf(output, "> pop pop\n"); // close and ignore the opened hex string
+      transformed_draw_extra(str, n, x, y, w, output, this, false);
+      return;
+    }
+    fprintf(output, "%4.4X", utf);
+  }
+  fprintf(output, "> %g %g show_pos_width\n", x, y);
+}
+
+void Fl_PostScript_Graphics_Driver::rtl_draw(const char* str, int n, int x, int y) {
+  int w = (int)width(str, n);
+  transformed_draw_extra(str, n, x - w, y, w, output, this, true);
+}
 
 void Fl_PostScript_Graphics_Driver::concat(){
   fprintf(output,"[%g %g %g %g %g %g] CT\n", fl_matrix->a , fl_matrix->b , fl_matrix->c , fl_matrix->d , fl_matrix->x , fl_matrix->y);
@@ -998,7 +1180,7 @@ void Fl_PostScript_Graphics_Driver::begin_points(){
   fprintf(output, "BP\n");
   gap_=1;
   shape_=POINTS;
-};
+}
 
 void Fl_PostScript_Graphics_Driver::begin_line(){
   fprintf(output, "GS\n");
@@ -1006,7 +1188,7 @@ void Fl_PostScript_Graphics_Driver::begin_line(){
   fprintf(output, "BP\n");
   gap_=1;
   shape_=LINE;
-};
+}
 
 void Fl_PostScript_Graphics_Driver::begin_loop(){
   fprintf(output, "GS\n");
@@ -1014,7 +1196,7 @@ void Fl_PostScript_Graphics_Driver::begin_loop(){
   fprintf(output, "BP\n");
   gap_=1;
   shape_=LOOP;
-};
+}
 
 void Fl_PostScript_Graphics_Driver::begin_polygon(){
   fprintf(output, "GS\n");
@@ -1022,7 +1204,7 @@ void Fl_PostScript_Graphics_Driver::begin_polygon(){
   fprintf(output, "BP\n");
   gap_=1;
   shape_=POLYGON;
-};
+}
 
 void Fl_PostScript_Graphics_Driver::vertex(double x, double y){
   if(shape_==POINTS){
@@ -1035,7 +1217,7 @@ void Fl_PostScript_Graphics_Driver::vertex(double x, double y){
     gap_=0;
   }else
     fprintf(output, "%g %g LT\n", x , y);
-};
+}
 
 void Fl_PostScript_Graphics_Driver::curve(double x, double y, double x1, double y1, double x2, double y2, double x3, double y3){
   if(shape_==NONE) return;
@@ -1046,7 +1228,7 @@ void Fl_PostScript_Graphics_Driver::curve(double x, double y, double x1, double
   gap_=0;
   
   fprintf(output, "%g %g %g %g %g %g curveto \n", x1 , y1 , x2 , y2 , x3 , y3);
-};
+}
 
 
 void Fl_PostScript_Graphics_Driver::circle(double x, double y, double r){
@@ -1062,7 +1244,7 @@ void Fl_PostScript_Graphics_Driver::circle(double x, double y, double r){
     
     fprintf(output, "%g %g %g 0 360 arc\n", x , y , r);
   
-};
+}
 
 void Fl_PostScript_Graphics_Driver::arc(double x, double y, double r, double start, double a){
   if(shape_==NONE) return;
@@ -1072,7 +1254,7 @@ void Fl_PostScript_Graphics_Driver::arc(double x, double y, double r, double sta
   else
     fprintf(output, "%g %g %g %g %g arcn\n", x , y , r , -start, -a);
   
-};
+}
 
 void Fl_PostScript_Graphics_Driver::arc(int x, int y, int w, int h, double a1, double a2) {
   fprintf(output, "GS\n");
@@ -1145,7 +1327,7 @@ void Fl_PostScript_Graphics_Driver::transformed_vertex(double x, double y){
   }else
     fprintf(output, "%g %g LT\n", x , y);
   concat();
-};
+}
 
 /////////////////////////////   Clipping /////////////////////////////////////////////
 
@@ -1218,7 +1400,7 @@ int Fl_PostScript_Graphics_Driver::clip_box(int x, int y, int w, int h, int &X,
     return 1;
   }
   return ret;
-};
+}
 
 int Fl_PostScript_Graphics_Driver::not_clipped(int x, int y, int w, int h){
   if(!clip_) return 1;
@@ -1227,7 +1409,7 @@ int Fl_PostScript_Graphics_Driver::not_clipped(int x, int y, int w, int h){
   clip_box(x, y, w, h, X, Y, W, H);
   if(W) return 1;
   return 0;
-};
+}
 
 
 void Fl_PostScript_File_Device::margins(int *left, int *top, int *right, int *bottom) // to implement
@@ -1248,6 +1430,11 @@ int Fl_PostScript_File_Device::printable_rect(int *w, int *h)
   return 0;
 }
 
+void Fl_PostScript_File_Device::origin(int *x, int *y)
+{
+  Fl_Paged_Device::origin(x, y);
+}
+
 void Fl_PostScript_File_Device::origin(int x, int y)
 {
   x_offset = x;
@@ -1259,6 +1446,7 @@ void Fl_PostScript_File_Device::origin(int x, int y)
 
 void Fl_PostScript_File_Device::scale (float s_x, float s_y)
 {
+  if (s_y == 0.) s_y = s_x;
   Fl_PostScript_Graphics_Driver *ps = driver();
   ps->scale_x = s_x;
   ps->scale_y = s_y;
@@ -1319,27 +1507,23 @@ void Fl_PostScript_File_Device::end_job (void)
   if(ferror(ps->output)) {
     fl_alert ("Error during PostScript data output.");
     }
-#if ! (defined(__APPLE__) || defined(WIN32) )
-  if (print_pipe)
-    pclose(ps->output);
-  else
+  if (ps->close_cmd_) {
+    (*ps->close_cmd_)(ps->output);
+  } else {
     fclose(ps->output);
-#else
-  fclose(ps->output);
-#endif
+    }
   while (ps->clip_){
     Fl_PostScript_Graphics_Driver::Clip * c= ps->clip_;
     ps->clip_= ps->clip_->prev;
     delete c;
   }
-  if (ps->close_cmd_) (*ps->close_cmd_)(ps->output);
   Fl_Display_Device::display_device()->set_current();
 }
 
 #if ! (defined(__APPLE__) || defined(WIN32) )
-int Fl_Printer::start_job(int pages, int *firstpage, int *lastpage) {
-  enum Fl_PostScript_Graphics_Driver::Page_Format format;
-  enum Fl_PostScript_Graphics_Driver::Page_Layout layout;
+int Fl_PostScript_Printer::start_job(int pages, int *firstpage, int *lastpage) {
+  enum Fl_Paged_Device::Page_Format format;
+  enum Fl_Paged_Device::Page_Layout layout;
 
   // first test version for print dialog
   if (!print_panel) make_print_panel();
@@ -1357,7 +1541,7 @@ int Fl_Printer::start_job(int pages, int *firstpage, int *lastpage) {
 
   // get options
 
-  format = print_page_size->value() ? Fl_PostScript_Graphics_Driver::A4 : Fl_PostScript_Graphics_Driver::LETTER;
+  format = print_page_size->value() ? Fl_Paged_Device::A4 : Fl_Paged_Device::LETTER;
   { // page range choice
     int from = 1, to = pages;
     if (print_pages->value()) {
@@ -1372,12 +1556,12 @@ int Fl_Printer::start_job(int pages, int *firstpage, int *lastpage) {
     pages = to - from + 1;
   }
   
-  if (print_output_mode[0]->value()) layout = Fl_PostScript_Graphics_Driver::PORTRAIT;
-  else if (print_output_mode[1]->value()) layout = Fl_PostScript_Graphics_Driver::LANDSCAPE;
-  else if (print_output_mode[2]->value()) layout = Fl_PostScript_Graphics_Driver::PORTRAIT;
-  else layout = Fl_PostScript_Graphics_Driver::LANDSCAPE;
+  if (print_output_mode[0]->value()) layout = Fl_Paged_Device::PORTRAIT;
+  else if (print_output_mode[1]->value()) layout = Fl_Paged_Device::LANDSCAPE;
+  else if (print_output_mode[2]->value()) layout = Fl_Paged_Device::PORTRAIT;
+  else layout = Fl_Paged_Device::LANDSCAPE;
 
-  print_pipe = print_choice->value();	// 0 = print to file, >0 = printer (pipe)
+  int print_pipe = print_choice->value();	// 0 = print to file, >0 = printer (pipe)
 
   const char *media = print_page_size->text(print_page_size->value());
   const char *printer = (const char *)print_choice->menu()[print_choice->value()].user_data();
@@ -1399,6 +1583,7 @@ int Fl_Printer::start_job(int pages, int *firstpage, int *lastpage) {
     fl_alert("could not run command: %s\n",command);
     return 1;
   }
+  ps->close_command(pclose);
   this->set_current();
   return ps->start_postscript(pages, format, layout); // start printing
 }
@@ -1408,5 +1593,5 @@ int Fl_Printer::start_job(int pages, int *firstpage, int *lastpage) {
 #endif // FL_DOXYGEN
 
 //
-// End of "$Id: Fl_PostScript.cxx 7653 2010-06-24 08:55:04Z manolo $".
+// End of "$Id: Fl_PostScript.cxx 8623 2011-04-24 17:09:41Z AlbrechtS $".
 //
