@@ -1,7 +1,7 @@
 /*
  * song.cpp - root of the model tree
  *
- * Copyright (c) 2004-2011 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2012 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -61,7 +61,6 @@
 #include "text_float.h"
 #include "timeline.h"
 
-
 tick_t midiTime::s_ticksPerTact = DefaultTicksPerTact;
 
 
@@ -81,6 +80,7 @@ song::song() :
 	m_modified( false ),
 	m_recording( false ),
 	m_exporting( false ),
+	m_exportLoop( false ),
 	m_playing( false ),
 	m_paused( false ),
 	m_loadingProject( false ),
@@ -343,7 +343,7 @@ void song::processNextBuffer()
 	bool check_loop = tl != NULL && m_exporting == false &&
 				tl->loopPointsEnabled() &&
 				!( m_playMode == Mode_PlayPattern &&
-					m_patternToPlay->freezing() == true );
+					m_patternToPlay->isFreezing() == true );
 	if( check_loop )
 	{
 		if( m_playPos[m_playMode] < tl->loopBegin() ||
@@ -467,11 +467,22 @@ void song::processNextBuffer()
 
 
 
+bool song::isFreezingPattern() const
+{
+	return isPlaying() &&
+				m_playMode == Mode_PlayPattern &&
+				m_patternToPlay != NULL &&
+				m_patternToPlay->isFreezing();
+}
+
+
+
+
 bool song::realTimeTask() const
 {
 	return !( m_exporting == true || ( m_playMode == Mode_PlayPattern &&
 		  	m_patternToPlay != NULL &&
-			m_patternToPlay->freezing() == true ) );
+			m_patternToPlay->isFreezing() == true ) );
 }
 
 
@@ -635,6 +646,7 @@ void song::stopExport()
 {
 	stop();
 	m_exporting = false;
+	m_exportLoop = false;
 }
 
 
@@ -1125,9 +1137,12 @@ void song::restoreControllerStates( const QDomElement & _this )
 }
 
 
+void song::exportProjectTracks()
+{
+	exportProject(true);
+}
 
-
-void song::exportProject()
+void song::exportProject(bool multiExport)
 {
 	if( isEmpty() )
 	{
@@ -1140,38 +1155,51 @@ void song::exportProject()
 	}
 
 	QFileDialog efd( engine::mainWindow() );
-	efd.setFileMode( QFileDialog::AnyFile );
-	efd.setAcceptMode( QFileDialog::AcceptSave );
-	int idx = 0;
-	QStringList types;
-	while( __fileEncodeDevices[idx].m_fileFormat !=
-					ProjectRenderer::NumFileFormats )
+	if (multiExport)
 	{
-		types << tr( __fileEncodeDevices[idx].m_description );
-		++idx;
-	}
-	efd.setFilters( types );
-
-	QString base_filename;
-	if( !m_fileName.isEmpty() )
-	{
-		efd.setDirectory( QFileInfo( m_fileName ).absolutePath() );
-		base_filename = QFileInfo( m_fileName ).completeBaseName();
+		efd.setFileMode( QFileDialog::Directory);
+		efd.setWindowTitle( tr( "Select directory for writing exported tracks..." ) );
+		if( !m_fileName.isEmpty() )
+		{
+			efd.setDirectory( QFileInfo( m_fileName ).absolutePath() );
+		}
 	}
 	else
 	{
-		efd.setDirectory( configManager::inst()->userProjectsDir() );
-		base_filename = tr( "untitled" );
+		efd.setFileMode( QFileDialog::AnyFile );
+		int idx = 0;
+		QStringList types;
+		while( __fileEncodeDevices[idx].m_fileFormat !=
+						ProjectRenderer::NumFileFormats )
+		{
+			types << tr( __fileEncodeDevices[idx].m_description );
+			++idx;
+		}
+		efd.setFilters( types );
+		QString base_filename;
+		if( !m_fileName.isEmpty() )
+		{
+			efd.setDirectory( QFileInfo( m_fileName ).absolutePath() );
+			base_filename = QFileInfo( m_fileName ).completeBaseName();
+		}
+		else
+		{
+			efd.setDirectory( configManager::inst()->userProjectsDir() );
+			base_filename = tr( "untitled" );
+		}
+		efd.selectFile( base_filename + __fileEncodeDevices[0].m_extension );
+		efd.setWindowTitle( tr( "Select file for project-export..." ) );
 	}
-	efd.selectFile( base_filename + __fileEncodeDevices[0].m_extension );
-	efd.setWindowTitle( tr( "Select file for project-export..." ) );
+
+	efd.setAcceptMode( QFileDialog::AcceptSave );
+
 
 	if( efd.exec() == QDialog::Accepted &&
 		!efd.selectedFiles().isEmpty() && !efd.selectedFiles()[0].isEmpty() )
 	{
 		const QString export_file_name = efd.selectedFiles()[0];
 		exportProjectDialog epd( export_file_name,
-						engine::mainWindow() );
+						engine::mainWindow(), multiExport );
 		epd.exec();
 	}
 }
