@@ -184,9 +184,12 @@ MainWindow::MainWindow( void ) :
 
 	m_updateTimer.start( 1000 / 20, this );	// 20 fps
 
-	// connect auto save
-	connect(&m_autoSaveTimer, SIGNAL(timeout()), this, SLOT(autoSave()));
-	m_autoSaveTimer.start(1000 * 60); // 1 minute
+	if( configManager::inst()->value( "ui", "enableautosave" ).toInt() )
+	{
+		// connect auto save
+		connect(&m_autoSaveTimer, SIGNAL(timeout()), this, SLOT(autoSave()));
+		m_autoSaveTimer.start(1000 * 60); // 1 minute
+	}
 }
 
 
@@ -255,6 +258,12 @@ void MainWindow::finalize( void )
 					engine::getSong(),
 					SLOT( exportProject() ),
 					Qt::CTRL + Qt::Key_E );
+	project_menu->addAction( embed::getIconPixmap( "project_export" ),
+					tr( "E&xport tracks..." ),
+					engine::getSong(),
+					SLOT( exportProjectTracks() ),
+					Qt::CTRL + Qt::SHIFT + Qt::Key_E );
+
 	project_menu->addSeparator();
 	project_menu->addAction( embed::getIconPixmap( "exit" ), tr( "&Quit" ),
 					qApp, SLOT( closeAllWindows() ),
