@@ -113,6 +113,7 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 	m_manualChPiano( configManager::inst()->value( "ui",
 					"manualchannelpiano" ).toInt() ),
 	m_smoothScroll( configManager::inst()->value( "ui", "smoothscroll" ).toInt() ),
+	m_enableAutoSave( configManager::inst()->value( "ui", "enableautosave" ).toInt() ),
 	m_oneInstrumentTrackWindow( configManager::inst()->value( "ui",
 					"oneinstrumenttrackwindow" ).toInt() ),
 	m_compactTrackButtons( configManager::inst()->value( "ui",
@@ -464,7 +465,7 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 	tabWidget * ui_fx_tw = new tabWidget( tr( "UI effects vs. "
 						"performance" ).toUpper(),
 								performance );
-	ui_fx_tw->setFixedHeight( 90 );
+	ui_fx_tw->setFixedHeight( 100 );
 
 	ledCheckBox * disable_ch_act_ind = new ledCheckBox(
 				tr( "Disable channel activity indicators" ),
@@ -491,6 +492,14 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 				this, SLOT( toggleSmoothScroll( bool ) ) );
 
 
+	ledCheckBox * autoSave = new ledCheckBox(
+			tr( "Enable auto save feature" ), ui_fx_tw );
+	autoSave->move( 10, 80 );
+	autoSave->setChecked( m_enableAutoSave );
+	connect( autoSave, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleAutoSave( bool ) ) );
+
+
 
 	perf_layout->addWidget( ui_fx_tw );
 	perf_layout->addStretch();
@@ -759,6 +768,8 @@ void setupDialog::accept()
 					QString::number( m_manualChPiano ) );
 	configManager::inst()->setValue( "ui", "smoothscroll",
 					QString::number( m_smoothScroll ) );
+	configManager::inst()->setValue( "ui", "enableautosave",
+					QString::number( m_enableAutoSave ) );
 	configManager::inst()->setValue( "ui", "oneinstrumenttrackwindow",
 					QString::number( m_oneInstrumentTrackWindow ) );
 	configManager::inst()->setValue( "ui", "compacttrackbuttons",
@@ -926,6 +937,14 @@ void setupDialog::toggleSmoothScroll( bool _enabled )
 
 
 
+void setupDialog::toggleAutoSave( bool _enabled )
+{
+	m_enableAutoSave = _enabled;
+}
+
+
+
+
 
 
 void setupDialog::toggleCompactTrackButtons( bool _enabled )
