@@ -38,6 +38,7 @@
 #include "song_editor.h"
 #include "automatable_slider.h"
 #include "combobox.h"
+#include "config_mgr.h"
 #include "cpuload_widget.h"
 #include "embed.h"
 #include "lcd_spinbox.h"
@@ -51,6 +52,7 @@
 #include "visualization_widget.h"
 #include "AudioDevice.h"
 #include "piano_roll.h"
+#include "config_mgr.h"
 
 
 
@@ -76,7 +78,8 @@ void positionLine::paintEvent( QPaintEvent * _pe )
 songEditor::songEditor( song * _song, songEditor * & _engine_ptr ) :
 	trackContainerView( _song ),
 	m_s( _song ),
-	m_scrollBack( false )
+	m_scrollBack( false ),
+	m_smoothScroll( configManager::inst()->value( "ui", "smoothscroll" ).toInt() )
 {
 	_engine_ptr = this;
 
@@ -87,8 +90,11 @@ songEditor::songEditor( song * _song, songEditor * & _engine_ptr ) :
 	setFocus();
 
 	// create time-line
-	m_timeLine = new timeLine( TRACK_OP_WIDTH +
-					DEFAULT_SETTINGS_WIDGET_WIDTH, 32,
+	int widgetTotal = configManager::inst()->value( "ui",
+							"compacttrackbuttons" ).toInt()==1 ?
+		DEFAULT_SETTINGS_WIDGET_WIDTH_COMPACT + TRACK_OP_WIDTH_COMPACT :
+		DEFAULT_SETTINGS_WIDGET_WIDTH + TRACK_OP_WIDTH;
+	m_timeLine = new timeLine( widgetTotal, 32,
 					pixelsPerTact(),
 					m_s->m_playPos[song::Mode_PlaySong],
 					m_currentPosition, this );
@@ -650,41 +656,60 @@ void songEditor::updateScrollBar( int _len )
 
 
 
-static inline void animateScroll( QScrollBar *scrollBar, int newVal )
+static inline void animateScroll( QScrollBar *scrollBar, int newVal, bool smoothScroll )
 {
-	// do smooth scroll animation using QTimeLine
-	QTimeLine *t = scrollBar->findChild<QTimeLine *>();
-	if( t == NULL )
+	if( smoothScroll == false )
 	{
-		t = new QTimeLine( 600, scrollBar );
-		t->setFrameRange( scrollBar->value(), newVal );
-		t->connect( t, SIGNAL( finished() ), SLOT( deleteLater() ) );
-
-		scrollBar->connect( t, SIGNAL( frameChanged( int ) ), SLOT( setValue( int ) ) );
-
-		t->start();
+		scrollBar->setValue( newVal );
 	}
 	else
 	{
-		// smooth scrolling is still active, therefore just update the end frame
-		t->setEndFrame( newVal );
+		// do smooth scroll animation using QTimeLine
+		QTimeLine *t = scrollBar->findChild<QTimeLine *>();
+		if( t == NULL )
+		{
+			t = new QTimeLine( 600, scrollBar );
+			t->setFrameRange( scrollBar->value(), newVal );
+			t->connect( t, SIGNAL( finished() ), SLOT( deleteLater() ) );
+
+			scrollBar->connect( t, SIGNAL( frameChanged( int ) ), SLOT( setValue( int ) ) );
+
+			t->start();
+		}
+		else
+		{
+			// smooth scrolling is still active, therefore just update the end frame
+			t->setEndFrame( newVal );
+		}
 	}
 }
 
 
 void songEditor::updatePosition( const midiTime & _t )
 {
+	int widgetWidth, trackOpWidth;
+	if( configManager::inst()->value( "ui", "compacttrackbuttons" ).toInt() )
+	{
+		widgetWidth = DEFAULT_SETTINGS_WIDGET_WIDTH_COMPACT;
+		trackOpWidth = TRACK_OP_WIDTH_COMPACT;
+	}
+	else
+	{
+		widgetWidth = DEFAULT_SETTINGS_WIDGET_WIDTH;
+		trackOpWidth = TRACK_OP_WIDTH;
+	}
+
 	if( ( m_s->isPlaying() && m_s->m_playMode == song::Mode_PlaySong 
 		  && m_timeLine->autoScroll() == timeLine::AutoScrollEnabled) ||
 							m_scrollBack == true )
 	{
-		const int w = width() - DEFAULT_SETTINGS_WIDGET_WIDTH
-							- TRACK_OP_WIDTH
+		const int w = width() - widgetWidth
+							- trackOpWidth
 							- 32;	// rough estimation for width of right scrollbar
 		if( _t > m_currentPosition + w * midiTime::ticksPerTact() /
 							pixelsPerTact() )
 		{
-			animateScroll( m_leftRightScroll, _t.getTact() );
+			animateScroll( m_leftRightScroll, _t.getTact(), m_smoothScroll );
 		}
 		else if( _t < m_currentPosition )
 		{
@@ -692,14 +717,14 @@ void songEditor::updatePosition( const midiTime & _t )
 				(int)( _t - w * midiTime::ticksPerTact() /
 							pixelsPerTact() ),
 									0 );
-			animateScroll( m_leftRightScroll, t.getTact() );
+			animateScroll( m_leftRightScroll, t.getTact(), m_smoothScroll );
 		}
 		m_scrollBack = false;
 	}
 
 	const int x = m_s->m_playPos[song::Mode_PlaySong].m_timeLine->
 							markerX( _t ) + 8;
-	if( x >= TRACK_OP_WIDTH + DEFAULT_SETTINGS_WIDGET_WIDTH-1 )
+	if( x >= trackOpWidth + widgetWidth -1 )
 	{
 		m_positionLine->show();
 		m_positionLine->move( x, 50 );
